/**@file

// fecConfigGenericFunctions.ctl
This library contains the generic function for the configuration of the package.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)
*/
//@{


/** Get Archives names (Bool, Analog and Event)
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the device dp
@param sDeviceType input, the device type
@param dsNames output, the archive names (Bool, Analog and Event)
*/
fecConfigGenericFunctions_getArchiveNames(string sDp, string sDeviceType, dyn_string &dsNames)
{
  int i, j, iLen, iLen_j;
  string sArchiveClass, sName;
  dyn_string dsDpe, dsTemp, dsCheck;       
    
  switch(sDeviceType)
  {
    case "IcemonFec":
      // put the DPE for the Bool archive, e.g.: ".ProcessInput...."+ DELIMITER
      dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
      // put the DPE for the Ana archive, e.g.: ".ProcessInput...."+ DELIMITER
      dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
      // put the DPE for the Event archive, e.g.: ".ProcessInput...."+ DELIMITER
      dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);      
      break;
    default:
      dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
      dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);      
      dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);      
      break;
  }
  iLen=dynlen(dsDpe);
  for (i=1; i<=iLen; i++)
  {
    if (dsDpe[i]!=UN_CONFIG_EXPORT_UNDEF)
    {
      dynClear(dsTemp);
      dynClear(dsCheck);
        
      dsTemp=strsplit(dsDpe[i], DELIMITER);
      iLen_j=dynlen(dsTemp);
        
      for (j=1; j<=iLen_j; j++)
      {
        sName="";
        sArchiveClass="";
          
        dpGet(sDp + dsTemp[j] + ":_archive.1._class", sArchiveClass);
    
        if(dpExists(sArchiveClass + ".general.arName"))
           dpGet(sArchiveClass + ".general.arName:_online.._value", sName);
         else
         {
           if (i==1)
            sName=UN_CONFIG_EXPORT_ARCHIVE_1;
          else if (i==2)
            sName=UN_CONFIG_EXPORT_ARCHIVE_2;
          else if (i==3)
            sName=UN_CONFIG_EXPORT_ARCHIVE_3;
          else
            sName=UN_CONFIG_EXPORT_UNDEF;  
         }
           
         dynAppend(dsCheck, sName);  
      }
        
      dynUnique(dsCheck);
        
      if (dynlen(dsCheck)==1)    //check if same archive for same type
        sName=dsCheck[1];  
      else
        sName=UN_CONFIG_EXPORT_UNDEF;  
    }
    else    //default
    {
      if (i==1)
        sName=UN_CONFIG_EXPORT_ARCHIVE_1;
      else if (i==2)
        sName=UN_CONFIG_EXPORT_ARCHIVE_2;
      else if (i==3)
        sName=UN_CONFIG_EXPORT_ARCHIVE_3;
      else
        sName=UN_CONFIG_EXPORT_UNDEF;  
    }
    dynAppend(dsNames, sName);  
  }
}
void fecGenericFunctions_setAddresses(string deviceDp, string alias, int driverNum)
{
  if (dpExists(FEC_CONFIG_DP))
  {
    string deviceName;
    string freeMemProperty, inactMemProperty, usedSwapProperty, uptimeProperty, cpuProperty, missingProcProperty, reachableProperty;
    dpGet(FEC_CONFIG_DP + ".cmw.deviceName", deviceName,
          FEC_CONFIG_DP + ".cmw.properties.freeMemory", freeMemProperty,
          FEC_CONFIG_DP + ".cmw.properties.inactiveMemory", inactMemProperty,
          FEC_CONFIG_DP + ".cmw.properties.usedSwap", usedSwapProperty,
          FEC_CONFIG_DP + ".cmw.properties.uptime", uptimeProperty,
          FEC_CONFIG_DP + ".cmw.properties.cpu", cpuProperty,
          FEC_CONFIG_DP + ".cmw.properties.reachable", reachableProperty,          
          FEC_CONFIG_DP + ".cmw.properties.missingProcesses", missingProcProperty);

    strreplace(freeMemProperty, "%HOST", strtoupper(alias));    
    strreplace(inactMemProperty, "%HOST", strtoupper(alias));
    strreplace(usedSwapProperty, "%HOST", strtoupper(alias));
    strreplace(uptimeProperty, "%HOST", strtoupper(alias));
    strreplace(cpuProperty, "%HOST", strtoupper(alias));    
    strreplace(reachableProperty, "%HOST", strtoupper(alias));  
    strreplace(missingProcProperty, "%HOST", strtoupper(alias));
    
    dyn_string exceptionInfo;
    unConfigCMW_setAddress(deviceDp +".ProcessInput.readings.freeMemory.value", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_FLOAT_TYPE, "", "", "",
                         deviceName, freeMemProperty, "value", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
    unConfigCMW_setAddress(deviceDp +".ProcessInput.readings.freeMemory.valid", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_BOOL_TYPE, "", "", "",
                         deviceName, freeMemProperty, "valid", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
        
    unConfigCMW_setAddress(deviceDp +".ProcessInput.readings.inactMemory.value", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_FLOAT_TYPE, "", "", "",
                         deviceName, inactMemProperty, "value", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
        unConfigCMW_setAddress(deviceDp +".ProcessInput.readings.inactMemory.valid", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_BOOL_TYPE, "", "", "",
                         deviceName, inactMemProperty, "valid", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);    
    
    unConfigCMW_setAddress(deviceDp +".ProcessInput.readings.usedSwap.value", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_LONGLONG_TYPE, "", "", "",
                         deviceName, usedSwapProperty, "value", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
    unConfigCMW_setAddress(deviceDp +".ProcessInput.readings.usedSwap.valid", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_BOOL_TYPE, "", "", "",
                         deviceName, usedSwapProperty, "valid", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
    
    unConfigCMW_setAddress(deviceDp +".ProcessInput.readings.cpu.value", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_FLOAT_TYPE, "", "", "",
                         deviceName, cpuProperty, "value", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);

    unConfigCMW_setAddress(deviceDp +".ProcessInput.readings.cpu.valid", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_BOOL_TYPE, "", "", "",
                         deviceName, cpuProperty, "valid", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);

    
    unConfigCMW_setAddress(deviceDp +".ProcessInput.processes.countMissing", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_LONG_TYPE, "", "", "",
                         deviceName, missingProcProperty, "value", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
    unConfigCMW_setAddress(deviceDp +".ProcessInput.processes.listMissing", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_STRING_TYPE, "", "", "",
                         deviceName, missingProcProperty, "valueDescription", true, exceptionInfo);

    unConfigCMW_setAddress(deviceDp +".ProcessInput.reachable.value", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_LONG_TYPE, "", "", "",
                         deviceName, reachableProperty, "value", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
    unConfigCMW_setAddress(deviceDp +".ProcessInput.reachable.valid", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_BOOL_TYPE, "", "", "",
                         deviceName, reachableProperty, "valid", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);

    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
    unConfigCMW_setAddress(deviceDp +".ProcessInput.processes.valid", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_BOOL_TYPE, "", "", "",
                         deviceName, missingProcProperty, "valid", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);    

    //polling for uptime
    unConfigCMW_setAddress(deviceDp +".ProcessInput.readings.uptime.value", driverNum, DPATTR_ADDR_MODE_INPUT_POLL, CMW_LONGLONG_TYPE, "60s,60000", "", "",
                         deviceName, uptimeProperty, "value", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
     unConfigCMW_setAddress(deviceDp +".ProcessInput.readings.uptime.valid", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_BOOL_TYPE, "", "", "",
                         deviceName, uptimeProperty, "valid", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
    
    //set pseudo.dev
    unConfigCMW_setAddress(deviceDp + ".ProcessInput.valid", driverNum, DPATTR_ADDR_MODE_INPUT_SPONT, CMW_BOOL_TYPE, "", "", "", 
                          deviceName, missingProcProperty + ":pseudo.dev", "contents", true, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
    
  }
}
void fecGenericFunctions_setAlarms(string deviceDp)
{
  dyn_string exceptionInfo;
  fwAlertConfig_setAnalog(deviceDp + ".ProcessInput.processes.countMissing", makeDynString("OK", "Missing processes from transfer.ref"), makeDynFloat(1), 
                          makeDynString("", FEC_CONFIG_DEFAULT_ALERT_CLASS), "", makeDynString(), "", exceptionInfo);
  if (dynlen(exceptionInfo) == 0)
  {
    fwAlertConfig_activate(deviceDp + ".ProcessInput.processes.countMissing", exceptionInfo);
  }
  else
    DebugN(exceptionInfo);
  
  fwAlertConfig_setDigital(deviceDp + ".ProcessInput.reachable.value", makeDynString("OK", "FEC not reachable!"), 
                          makeDynString("", FEC_CONFIG_DEFAULT_ALERT_CLASS), "", makeDynString(), "", exceptionInfo);
  if (dynlen(exceptionInfo) == 0)
  {
    fwAlertConfig_activate(deviceDp + ".ProcessInput.reachable.value", exceptionInfo);
  }
  else
    DebugN(exceptionInfo);

  
  dyn_string summAlertDps;
  dynAppend(summAlertDps, deviceDp + ".ProcessInput.processes.countMissing");
  dynAppend(summAlertDps, deviceDp + ".ProcessInput.reachable.value");  
  fwAlertConfig_setSummary(deviceDp + ".ProcessInput.alert", makeDynString(UN_CONFIG_DEFAULT_ALERT_DIGITAL_TEXT_OK, UN_CONFIG_DEFAULT_ALERT_DIGITAL_TEXT_BAD),
                           summAlertDps, "", makeDynString(), "", exceptionInfo);
  if (dynlen(exceptionInfo) == 0)
    fwAlertConfig_activate(deviceDp + ".ProcessInput.alert", exceptionInfo);
  else 
    DebugN(exceptionInfo);
  
  if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
}
void fecGenericFunctions_connectValidityDps(string deviceDp)
{
  dyn_string exceptionInfo;
  fwDpFunction_setDpeConnection(deviceDp + ".ProcessInput.publicationsValid",
                                makeDynString(deviceDp + ".ProcessInput.readings.freeMemory.valid:_online.._value",
                                              deviceDp + ".ProcessInput.readings.inactMemory.valid:_online.._value",
                                              deviceDp + ".ProcessInput.readings.usedSwap.valid:_online.._value",
                                              deviceDp + ".ProcessInput.readings.uptime.valid:_online.._value",
                                              deviceDp + ".ProcessInput.readings.cpu.valid:_online.._value",
                                              deviceDp + ".ProcessInput.processes.valid:_online.._value",
                                              deviceDp + ".ProcessInput.reachable.valid:_online.._value"),
                                makeDynString(),
                                "p1&&p2&&p3&&p4&&p5&&p6&&p7",
                                exceptionInfo, 
                                1);
   if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
                                                  
}
void fecGenericFunctions_setArchiving(string deviceDp, string archive)
{
  dyn_string exceptionInfo;  
  dyn_string dpes;
  dynAppend(dpes, deviceDp + ".ProcessInput.readings.freeMemory.value");   
  dynAppend(dpes, deviceDp + ".ProcessInput.readings.inactMemory.value");   
  dynAppend(dpes, deviceDp + ".ProcessInput.readings.usedSwap.value");   
  dynAppend(dpes, deviceDp + ".ProcessInput.readings.cpu.value");     
  fwArchive_setMultiple(dpes, archive, DPATTR_ARCH_PROC_SIMPLESM, DPATTR_TIME_OR_VALUE_REL_SMOOTH, 1, 60, exceptionInfo, TRUE);  
  if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
  
  dyn_string dpesOldNew;
  dynAppend(dpesOldNew, deviceDp + ".ProcessInput.processes.countMissing");   
  dynAppend(dpesOldNew, deviceDp + ".ProcessInput.processes.listMissing");     
  dynAppend(dpesOldNew, deviceDp + ".ProcessInput.processes.valid");     
  dynAppend(dpesOldNew, deviceDp + ".ProcessInput.reachable.value"); 
  dynAppend(dpesOldNew, deviceDp + ".ProcessInput.reachable.valid");
  dynAppend(dpesOldNew, deviceDp + ".ProcessInput.readings.freeMemory.valid");   
  dynAppend(dpesOldNew, deviceDp + ".ProcessInput.readings.inactMemory.valid");   
  dynAppend(dpesOldNew, deviceDp + ".ProcessInput.readings.usedSwap.valid");   
  dynAppend(dpesOldNew, deviceDp + ".ProcessInput.readings.cpu.valid");   
  fwArchive_setMultiple(dpesOldNew, archive, DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0, 0, exceptionInfo, TRUE);
  if (dynlen(exceptionInfo) > 0) DebugN(exceptionInfo);
}
//@}
