/**@file

// fecGenericFunctions.ctl
This library contains the generic functions of the package.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)
*/
//@{


/** add/reset default user, password not change if already set.
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
fecGenericFunctions_addDefaultUser()
{
  string acComponent="fwAccessControl";
  string acVersion="3.2.7", sPassword;
  int acInstalled, iPos;
  dyn_string exceptionInfo, dsUserName, dsPassword;
  bool bIntegratedMode;
  
  fwInstallation_componentInstalled(acComponent, acVersion, acInstalled);
  if (!acInstalled)
  {
    DebugTN("fecGenericFunctions_setDefaultUser: Access Control component " + acVersion + " not installed!");
    return;
  }

  bIntegratedMode=_fwAccessControl_integratedMode();
  if(!bIntegratedMode)
  {
    // use here the fwAccessControl functions to create domains, privileges, groups, users
    DebugTN("fecGenericFunctions_setDefaultUser: FEC domain, group and user updated");
  }
  else
    DebugTN("fecGenericFunctions_setDefaultUser: fwAccessControl mode activated, the changes are automatically populated from the AC server.");
  
}

/** set the text and the colors of the header in the faceplate status tab (state of the FE connection)
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sFEDp  input, front-end DP name
@param sFEAlias  input, front-end name (alias)
@param sDeviceAlias  input, device name (alias)
@param iAlarm  input, front-end comm system integrity alarm level
@param bSystemAlarmEnable  input, front-end comm system integrity enable/disable
@param bInvalid  input, device invalid state
*/
fecGenericFunctions_FaceplateStatusHeader(string sFEDp, string sFEAlias, string sDeviceAlias, int iAlarm, bool bSystemAlarmEnable, bool bInvalid)
{
  bool bOk=false, bFrontEnd=false;
  time tLast;
  string sBackColor, sForeColor, sText, sTime, sName;

  sName=sFEAlias;
  if (sName=="")
    sName="?";
  else if (sName!=sFEDp)
    bFrontEnd=true;

  if(bInvalid)
  {
    sBackColor="unDataNotValid";
    sForeColor="_3DText";
    sTime="";
    sText="DATA INVALID";
  }
  else if(!bSystemAlarmEnable)
  {
    //SystemAlarm disabled
    //sBackColor="unAlarmMasked";
    sBackColor="unDataNotValid";  
    sForeColor="_3DText";
    sTime="";
    sText=sName + " DATA Connection DISABLED";
  }
  else
  {
    if(iAlarm > c_unSystemIntegrity_no_alarm_value)
    {
      //Connection stopped
      sBackColor="unDataNotValid";
      sForeColor="_3DText";
      sText=sName + " NO DATA Connection";
   
      if(bFrontEnd)
      {
        if (dpExists(sFEDp))
        {
         dpGet(sFEDp + ".communication.counter:_online.._stime", tLast);
         sTime=" since: " + (string)tLast; 
        }
        else
        {
         sTime="?";
        }  
      }
      else
        sTime="";
    }
    else
    {
      //Connection FE OK
      sBackColor="_3DFace";
      sForeColor="_3DText";
      sText=sName + " DATA Connection OK";
      sTime="";   
      bOk=true;
    }
  }
  sText=sText + sTime;
  if(shapeExists("txtFEstatus"))
  {
    setMultiValue("txtFEstatus", "backCol", sBackColor,
                  "txtFEstatus", "foreCol", sForeColor,
                  "txtFEstatus", "text", sText);
  }
}

/** set the text and the colors of the header in the faceplate status tab (state of the FE connection) in 
  case of disconnection of the remote system
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
void fecGenericFunctions_FaceplateStatusDisconnect()
{
  string sBackColor, sForeColor, sText;

  sBackColor="_3DFace";
  sForeColor="unDataNoAccess";
  sText="DATA not connected";
    
  if(shapeExists("txtFEstatus"))
  {
    setMultiValue("txtFEstatus", "backCol", sBackColor,
                  "txtFEstatus", "foreCol", sForeColor,
                  "txtFEstatus", "text", sText);
  }
}
void fecGenericFunctions_animateWidgetsValidity(string sDpName, bool valid)
{
  string okColor = "_Window";
  string invalidColor = "_invalid";
  string freeMemColor, inactMemColor, uptimeColor, swapColor, processesColor, reachableColor, cpuColor;
  bool freeMemValid, inactMemValid, uptimeValid, swapValid, processesValid, reachableValid, cpuValid;
  bool processesAlarmActive, reachableAlarmActive;
  if (!valid)
  {
    dpGet(sDpName + ".ProcessInput.readings.freeMemory.valid", freeMemValid,
          sDpName + ".ProcessInput.readings.inactMemory.valid", inactMemValid,
          sDpName + ".ProcessInput.readings.uptime.valid", uptimeValid,
          sDpName + ".ProcessInput.readings.usedSwap.valid", swapValid,
          sDpName + ".ProcessInput.processes.valid", processesValid,
          sDpName + ".ProcessInput.reachable.valid", reachableValid,
          sDpName + ".ProcessInput.readings.cpu.valid",  cpuValid,
          sDpName + ".ProcessInput.processes.countMissing:_alert_hdl.._act_state", processesAlarmActive,
          sDpName + ".ProcessInput.reachable.value:_alert_hdl.._act_state", reachableAlarmActive);
  }
  else
  {
    freeMemValid = inactMemValid = uptimeValid = swapValid = processesValid = reachableValid = cpuValid = true;
    dpGet(sDpName + ".ProcessInput.processes.countMissing:_alert_hdl.._act_state", processesAlarmActive,
          sDpName + ".ProcessInput.reachable.value:_alert_hdl.._act_state", reachableAlarmActive);
  }
  freeMemColor = freeMemValid?okColor:invalidColor;
  inactMemColor = inactMemValid?okColor:invalidColor;
  uptimeColor = uptimeValid?okColor:invalidColor;
  swapColor = swapValid?okColor:invalidColor;
  processesColor = processesValid?okColor:invalidColor;
  reachableColor = reachableValid?"_3DFace":invalidColor;
  cpuColor = cpuValid?okColor:invalidColor;
  string fecReachableText = reachableValid?"":"FEC reachable?";
  setMultiValue("tfUptime", "backCol", uptimeColor,
                 "tfFreeMemory", "backCol", freeMemColor,
                 "tfInactiveMemory", "backCol", inactMemColor,
                 "tfUsedSwap", "backCol", swapColor,
                 "tfCPU", "backCol", cpuColor);
  if (!reachableAlarmActive || !reachableValid)
  {
    setMultiValue("tfFecReachable", "text", fecReachableText,
                 "tfFecReachable", "backCol", reachableColor);
  }
  
  if (!processesAlarmActive || !processesValid)
  {
    setMultiValue( "tfMissingProcesses", "backCol", processesColor,
                 "tblMissingProcesses", "backCol", processesColor);
  }
}
//@}
