/**@file

// fecIcemonFec.ctl
This library contains the widget, faceplate, etc. functions of IcemonFec.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)
*/
//@{


/** Returns the list of IcemonFec DPE with alarm config that can be acknowledged
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName  input, device name DP name
@param sDpType input, device type
@param dsNeedAck output, the lsit of DPE
*/
fecIcemonFec_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
  dsNeedAck = makeDynString("ProcessInput....");
  // if the acknoledge must be sent to the PLC: CPC Front-end: add UN_ACKNOWLEDGE_PLC
  //dsNeedAck = makeDynString("ProcessInput....", UN_ACKNOWLEDGE_PLC);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec AcknowledgeAlarm IcemonFec "+sDpName, "fecIcemonFec_AcknowledgeAlarm", sDpName, sDpType, dsNeedAck);
}

/** Function called from snapshot utility of the treeDeviceOverview to get the time and value
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDeviceName  input, device name
@param sDeviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
fecIcemonFec_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
  float freeMem, usedSwap, inactMem; 
  int missingProcessesCount, uptime;
 
  dpGet(sDeviceName+".ProcessInput.readings.freeMemory", freeMem, 
        sDeviceName+".ProcessInput.readings.inactMemory", inactMem, 
        sDeviceName+".ProcessInput.readings.usedSwap", usedSwap, 
        sDeviceName+".ProcessInput.readings.uptime", uptime, 
        sDeviceName+".ProcessInput.processes.countMissing", missingProcessesCount
        );
  dsReturnData = makeDynString(freeMem, inactMem, usedSwap, uptime, missingProcessesCount);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec snapshot IcemonFec "+sDeviceName, "fecIcemonFec_ObjectListGetValueTime", sDeviceName, dsReturnData);
}

/** faceplate DistributedControl callback of the faceplate status panel
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
fecIcemonFec_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, sSystemName;
  int iAction, iRes;
  dyn_string exceptionInfo;
  bool bRemote;
  
  deviceName = unGenericDpFunctions_getDpName($sDpName);
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
  unDistributedControl_isRemote(bRemote, sSystemName);
  if(bRemote)
    g_bSystemConnected = bSystemConnected;
  else
    g_bSystemConnected = true;
                                         
  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec faceplate IcemonFec "+sDp, "fecIcemonFec_FaceplateStatusRegisterCB", sDp, deviceName, g_bCallbackConnected, iAction);
  switch (iAction)
  {
    case UN_ACTION_DISCONNECT:
      fecIcemonFec_FaceplateStatusDisconnection();
      break;
    case UN_ACTION_DPCONNECT:
      fecIcemonFec_FaceplateConnect(deviceName);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      fecIcemonFec_FaceplateDisconnect(deviceName);
      fecIcemonFec_FaceplateStatusDisconnection();
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
  }
}

/** set the faceplate when the device system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
fecIcemonFec_FaceplateStatusDisconnection()
{
// 2. animate FE Connection
  fecGenericFunctions_FaceplateStatusDisconnect();
  // there is no need to do here execScript because there is one faceplate per device type
  // set all the graphical element to the state data not connnected
  // ex. of functions that can be used
  //unGenericObject_ColorBoxDisconnect, unGenericObject_DisplayValueDisconnect, etc.
  unGenericObject_DisplayValueDisconnect(makeDynString("tfFreeMemory", "tfUptime", "tfInactMemory", "tfUsedSwap", "tfMissingProcesses"));
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec faceplate IcemonFec", "fecIcemonFec_FaceplateStatusDisconnection");
}

/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
*/
fecIcemonFec_FaceplateConnect(string deviceName)
{
  int iAction, iRes;
  dyn_string exceptionInfo;
  string sFrontEnd, sSystemName;
  
  g_sDeviceAlias = unGenericDpFunctions_getAlias(deviceName);

  g_bUnSystemAlarmPlc = false;
// get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
  g_sFrontEndDp = sSystemName+sFrontEnd;
  g_sFrontEndAlias = unGenericDpFunctions_getAlias(g_sFrontEndDp);
  
  if(sFrontEnd != ""){
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
    if(dpExists(sFrontEnd)) {
      g_bUnSystemAlarmPlc = true;
    }
  }
    
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec faceplate IcemonFec "+deviceName, "fecIcemonFec_FaceplateConnect", deviceName, sFrontEnd, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc) {    
    // set all faceplate global variable usefull for the callback, e.g.: description, format, unit, etc.
    // set graphical element of the faceplate to the init state if needed

    
    // if there are many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // keep the kind of dpConnect in a variable global to the faceplate and do a swith for the dpConnect
    // and do to the correct dpDisconnect in the fecIcemonFec_FaceplateDisconnect
    iRes = dpConnect("fecIcemonFec_FaceplateStatusAnimationCB",
                      deviceName + ".ProcessInput.valid",
                      deviceName + ".ProcessInput.publicationsValid",                                            
                      deviceName + ".ProcessInput.reachable.value",                      
                      deviceName + ".ProcessInput.readings.freeMemory.value",
                      deviceName + ".ProcessInput.readings.inactMemory.value",                      
                      deviceName + ".ProcessInput.readings.usedSwap.value",                      
                      deviceName + ".ProcessInput.readings.cpu.value",                        
                      deviceName + ".ProcessInput.readings.uptime.value",                      
                      deviceName + ".ProcessInput.processes.countMissing",                      
                      deviceName + ".ProcessInput.processes.listMissing",                                            
                      sFrontEnd+".alarm",
                      sFrontEnd+".enabled");
    g_bCallbackConnected = (iRes >= 0);
  }
  else
    fecIcemonFec_FaceplateStatusDisconnection();
}

/** dpDisconnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
*/
fecIcemonFec_FaceplateDisconnect(string deviceName)
{
  int iRes;
  string sFrontEnd, sSystemName;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec faceplate IcemonFec "+deviceName, "fecIcemonFec_FaceplateDisconnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc) {
// get the systemName
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
    sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;

    // if there were many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // re-use the kind of dpConnect kept in a variable global to the faceplate set during fecIcemonFec_FaceplateConnect
    // and do to the correct dpDisconnect
    iRes = dpDisconnect("fecIcemonFec_FaceplateStatusAnimationCB",
                      deviceName + ".ProcessInput.valid",
                      deviceName + ".ProcessInput.publicationsValid",                      
                      deviceName + ".ProcessInput.reachable.value",                      
                      deviceName + ".ProcessInput.readings.freeMemory.value",
                      deviceName + ".ProcessInput.readings.inactMemory.value",                      
                      deviceName + ".ProcessInput.readings.usedSwap.value",                      
                      deviceName + ".ProcessInput.readings.cpu.value",                        
                      deviceName + ".ProcessInput.readings.uptime.value",                      
                      deviceName + ".ProcessInput.processes.countMissing",                      
                      deviceName + ".ProcessInput.processes.listMissing",                                            
                      sFrontEnd+".alarm",
                      sFrontEnd+".enabled");
    g_bCallbackConnected = !(iRes >= 0);
  }
}

/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpValid input, the valid DPE 
@param bValid input, data validity
@param sDpFESystemIntegrityAlarmValue input, front-end device system integrity alarm DPE 
@param iFESystemIntegrityAlarmValue input, front-end device system integrity alarm value
@param sDpFESystemIntegrityAlarmEnabled input, front-end device system integrity enable DPE
@param bFESystemIntegrityAlarmEnabled input, front-end device system integrity enable value
*/
fecIcemonFec_FaceplateStatusAnimationCB(string sDpValid, bool bValid,
                                        string sDpPublicationsValid, bool publicationsValid,
                                        string sDpReachable, bool notReachable,
                                        string sDpFreeMemoty, float freeMemory,
                                        string sDpInactMemory, float inactMemory,
                                        string sDpUsedSwap, float usedSwap,
                                        string sDpCPU, float cpu,                                        
                                        string sDpUptime, int uptime,
                                        string sDpMissingCount, int countMissing,
                                        string sDpMissingList, string listMissing,                                        
                                        string sDpFESystemIntegrityAlarmValue, int iFESystemIntegrityAlarmValue, 
                                        string sDpFESystemIntegrityAlarmEnabled, bool bFESystemIntegrityAlarmEnabled)
{
  if(g_bSystemConnected)
    fecGenericFunctions_FaceplateStatusHeader(g_sFrontEndDp, g_sFrontEndAlias, g_sDeviceAlias, iFESystemIntegrityAlarmValue, 
                                              bFESystemIntegrityAlarmEnabled, !bValid);
  
	string sColor = "green";
	if (!bValid)
	{
    sColor = "_invalid";
	}
  else if (countMissing > 0)
  {
    sColor = "red";
  }
  string  sBackColor="_3DFace";
  string reachcableText="";
  if (notReachable)
  {
    sBackColor="red";
    reachcableText = "FEC not reachable!";
  }
  if(g_bSystemConnected)
  {
    	setMultiValue("tfUptime", "text", formatTime("%d/%m/%y %H:%M", getCurrentTime() - (time)uptime),
                   "tfFreeMemory", "text", floor(freeMemory) + "%",
                   "tfInactiveMemory", "text", floor(inactMemory) + "%",
                   "tfUsedSwap", "text", floor(usedSwap) + "%",
                   "tfCPU", "text", floor(cpu) + "%",                   
                   "tfMissingProcesses", "text", countMissing,
                   "tfMissingProcesses", "backCol", sColor,
                   "tfFecReachable", "text", reachcableText,
                   "tfFecReachable", "backCol", sBackColor);

      if (countMissing ==0)
      {
        tblMissingProcesses.deleteAllLines();
      }
      else
      {
        dyn_string missing = strsplit(listMissing, ",");
        tblMissingProcesses.appendLines(dynlen(missing), "processName", missing);
      }
      string dp = dpSubStr(sDpValid, DPSUB_SYS_DP);
      fecGenericFunctions_animateWidgetsValidity(dp, publicationsValid);
  }
}

/** Contextual button DistributedControl callback of the contextual device button panel
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
fecIcemonFec_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, sDpType, sFunction;
  int iAction, iRes, iManReg01;
  dyn_string exceptionInfo, dsFunctions, dsUserAccess;
  
  deviceName = unGenericDpFunctions_getDpName($sDpName);
  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec button IcemonFec "+deviceName, "fecIcemonFec_ButtonRegisterCB", sDp, deviceName, g_bCallbackConnected, iAction);
  switch (iAction)
  {
    case UN_ACTION_DISCONNECT:
      fecIcemonFec_ButtonDisconnection();
      break;
    case UN_ACTION_DPCONNECT:

      sDpType="IcemonFec";
      unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
      sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
             
      if (sFunction != "")
      {
        evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
               "dyn_string dsUserAccessReturn;" +
               "if (isFunctionDefined(\"" + sFunction  + "\"))" +
               "    {" +
                 sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
               "    }" +
               "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
        // keep the list of the device allowed action 
        g_dsUserAccess=dsUserAccess;  
      }
      else
        fwException_raise(exceptionInfo,"ERROR", "fecIcemonFec_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");

    // set all contextual panel global variable usefull for the fecIcemonFec_ButtonAnimationCB, e.g.: description, format, unit, etc.
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec button IcemonFec "+deviceName, "fecIcemonFec_ButtonRegisterCB Connect", deviceName, sDpType, dsUserAccess);
      iRes = dpConnect("fecIcemonFec_ButtonAnimationCB",
               deviceName + ".statusInformation.selectedManager:_lock._original._locked",
               deviceName + ".statusInformation.selectedManager");
      g_bCallbackConnected = (iRes >= 0);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec button IcemonFec "+deviceName, "fecIcemonFec_ButtonRegisterCB Disconnect", deviceName);
      iRes = dpDisconnect("fecIcemonFec_ButtonAnimationCB",
               deviceName + ".statusInformation.selectedManager:_lock._original._locked",
               deviceName + ".statusInformation.selectedManager");
      g_bCallbackConnected = !(iRes >= 0);
      fecIcemonFec_ButtonDisconnection();
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
  }
}

/** return the device data, this function is called when the user logs in and logs out
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param sType input, the device type
@param dsReturnData output, the device data, [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
fecIcemonFec_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
  int iRes;
  string sDpLock, sDpSelectedManager, sSelectedManager, dpData;
  bool bLocked;

  // read the device data (6 string)
  sDpLock = deviceName + ".statusInformation.selectedManager:_lock._original._locked";
  sDpSelectedManager = deviceName + ".statusInformation.selectedManager";

  if (dpExists(deviceName))
  {
    iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager);
  }
  else
  {
    iRes = -1;
  }
  if (iRes < 0) // Errors during dpget
  {
    bLocked = false;
    sSelectedManager = "";
  }
  dsReturnData=makeDynString(bLocked, sSelectedManager, "", "", "", "");
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec button IcemonFec "+deviceName, "fecIcemonFec_UserLoginGetButtonState", deviceName, sType, dsReturnData);
}

/** Set the state of the contextual button of the device
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param sDpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
fecIcemonFec_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
  bool bSelected;
  dyn_string dsButtons;
  int i, length;
  string localManager;
  bool bLocked, buttonEnabled;
  string sSelectedManager, sDeviceValue1, sDeviceValue2, sDeviceValue3, sDeviceValue4;

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec button IcemonFec "+deviceName, "fecIcemonFec_ButtonSetState", deviceName, sDpType, dsUserAccess, dsData);
  bLocked = dsData[1];
  sSelectedManager = dsData[2];
  sDeviceValue1 = dsData[3];
  sDeviceValue2 = dsData[4];
  sDeviceValue3 = dsData[5];
  sDeviceValue4 = dsData[6];

  // dsButtons = list of all the possible actions except Select 
  // use the name of the graphical element without "button"
  // use the name of the graphical elements.
  // put here the list of all the device actions (list of all the buttons)
  dsButtons = makeDynString("Reset", "Increase", "Decrease", "Copy", "Info", "FECnt", "Alerts"); 
  length = dynlen(dsButtons);
// 1. Selection state
  localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
  bSelected = (localManager == "S");
// 2. for each possible action check if the action is in the list dsUserAccess
  // crosscheck if needed with the state of the device
  for(i=1;i<=length;i++)
  {
    buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);  // User access
    switch(dsButtons[i])
    {
      case "Reset":
        // calculate the enable state of the button from the device data
        buttonEnabled = buttonEnabled & bSelected & (sDeviceValue1 == "10");
        break;
      case "Increase":
      case "Decrease":
      case "Copy":
        // calculate the enable state of the button from the device data
        buttonEnabled = buttonEnabled & bSelected;
        break;
      case "Info":
      case "FECnt":
        break;
      // add here all the device actions if needed
      case "Alerts":
        buttonEnabled = buttonEnabled & bSelected;
        break;
      default:
        break;
    }

    setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
  }
// 3. Button Select state
  unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

/** set the state of the device action button when the device system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
fecIcemonFec_ButtonDisconnection()
{
  // disable all the device action (button)
//  unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_SELECT, ".."));
  unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_SELECT, "Reset", "Increase", "Decrease", "Copy", "Info", "FECnt", "Alerts"));
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec button IcemonFec", "fecIcemonFec_ButtonDisconnection");
}

/** returns the list of allowed action on the device for a user logged in
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName input, the device DP name
@param sDpType input, the device type
@param dsAccess input, list of allowed action on the device
*/
fecIcemonFec_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
  bool operator, expert, admin;
  int i, length;
  dyn_bool dbPermissions;
  dyn_string exceptionInfo;
  string sOperAction, sExpAction, sAdminAction;
  bool bActionDefined = false;

  // get the configured list of authorized action per privilege
  unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
  if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
     (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
    bActionDefined=true;
  operator = dbPermissions[2];
  expert = dbPermissions[3];
  admin = dbPermissions[4];
  dsAccess = makeDynString();
  
  // for each privilege get the list of allowed actions if there are action defined on the device
  // if there are no action defined for the device get the default one
  // in the example, the default behavior is 
  // the FECnt device action is always allowed
  // the Info device action (button) is allowed if one has the operator privilege
  // the Select, Increase and Decrease device actions (buttons) are allowed if one has the expert privilege
  // the Copy and Reset button device actions (buttons) are allowed if one has the admin privilege
  dynAppend(dsAccess, makeDynString("FECnt")); // put here the devcie action that always allowed
  if (operator)
  {
    if(bActionDefined)
      dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
    else
      dynAppend(dsAccess, makeDynString("Info")); // put here all the list of device actions (buttons) allowed with the operator privilege
  }
  if (expert)
  {
    if(bActionDefined)
      dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
    else 
      dynAppend(dsAccess, makeDynString("Increase", "Decrease", UN_FACEPLATE_BUTTON_SELECT)); // put here all the list of device actions (buttons) allowed with the expert privilege
  }
  if(admin){
    if(bActionDefined)
      dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
    else
      dynAppend(dsAccess, makeDynString("Copy", "Reset", "Alerts")); // put here all the list of device actions (buttons) allowed with the admin privilege
  }
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec button IcemonFec "+sDpName, "fecIcemonFec_ButtonUserAccess", sDpName, sDpType, dbPermissions, sOperAction, sExpAction, sAdminAction, dsAccess);
}

/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpLock input, the lock DPE 
@param bLocked input, selected lock state of the device
@param sDpSelect input, the select DPE 
@param sSelectedManager input, user and Ui that selected the device
*/
fecIcemonFec_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager)
{
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec button IcemonFec "+sDpLock, "fecIcemonFec_ButtonAnimationCB", g_dsUserAccess);
  // call the set button state with the device data and the current allowed action on the device
  // which is in g_dsUserAccess
  fecIcemonFec_ButtonSetState(unGenericDpFunctions_getDpName(sDpLock), "IcemonFec", g_dsUserAccess, 
                  makeDynString(bLocked, sSelectedManager, "Alerts", "", "", ""));
}

/** widget DistributedControl callback
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
fecIcemonFec_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, sSystemName;
  int iAction;
  dyn_string exceptionInfo;
  string sFrontEnd;
  bool bRemote;
  
  deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
  if (deviceName == "")  // In case of disconnection
  {
    deviceName = g_sDpName;
  }
  g_bUnSystemAlarmPlc = false;
// get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);

  unDistributedControl_isRemote(bRemote, sSystemName);
  if(bRemote)
    g_bSystemConnected = bSystemConnected;
  else
    g_bSystemConnected = true;

  if(sFrontEnd != ""){
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
    if(dpExists(sFrontEnd)) {
      g_bUnSystemAlarmPlc = true;
    }
  }

  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec widget IcemonFec "+deviceName, "fecIcemonFec_WidgetRegisterCB", sDp, deviceName, g_bCallbackConnected, iAction);
  
  //resolve FSM dp
  string fsmDp;
  if (isDollarDefined("$1") && isDollarDefined("$2") && $1 != "" && $2 != "")
  {
    fsmDp = getSystemName() + $1 + "|" + $2;
  } 
  else
  {
    string alias = unGenericDpFunctions_getAlias(deviceName);
    dyn_string fsmDps = dpNames("*|" + alias, "_FwFsmDevice");
    if (dynlen(fsmDps) > 0)
      fsmDp = fsmDps[1];
  }
  switch(iAction)
  {
    case UN_ACTION_DISCONNECT:
      fecIcemonFec_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_DPCONNECT:
      g_sDpName = deviceName;
      fecIcemonFec_WidgetConnect(deviceName, sFrontEnd, fsmDp);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      fecIcemonFec_WidgetDisconnect(deviceName, sFrontEnd, fsmDp);
      fecIcemonFec_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
  }
}

/** set the widget when the system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sWidgetType input, the type of widget
*/
fecIcemonFec_WidgetDisconnection(string sWidgetType)
{
  /** use the exec mechanism to allow multiple widget. **/
  string sFunction = "fecIcemonFec_"+sWidgetType+"Disconnection";

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec widget IcemonFec", "fecIcemonFec_WidgetDisconnection", sWidgetType);
  if (isFunctionDefined(sFunction) && (sFunction != ""))
  {
    execScript("main(string sWidgetType) {" + sFunction + "(sWidgetType);}", makeDynString(), sWidgetType);
  }
}

/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, front-end device DP name
@param sFrontEnd input, front-end device name
*/
fecIcemonFec_WidgetConnect(string deviceName, string sFrontEnd, string fsmDp)
{
  int iRes;
  string sSystemName;
  
//  check if the device name is of type "IcemonFec" if not like no front-end. -> not available.
  if(dpTypeName(deviceName) != "IcemonFec")
    g_bUnSystemAlarmPlc = false;
  
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec widget IcemonFec "+deviceName, "fecIcemonFec_WidgetConnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc)
  {
    // if there are many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // keep the kind of dpConnect in a variable global to the widget and do a swith for the dpConnect
    // and do to the correct dpDisconnect in the fecIcemonFec_WidgetDisconnect
    iRes = dpConnect("fecIcemonFec_WidgetCB", 
                          deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                          deviceName + ".statusInformation.selectedManager",
                          fsmDp + ".fsm.currentState",
                          deviceName + ".:_alert_hdl.._act_state",
                          deviceName + ".ProcessInput.readings.freeMemory.value",
                          deviceName + ".ProcessInput.readings.usedSwap.value",
                          deviceName + ".ProcessInput.readings.cpu.value",                          
                          deviceName + ".ProcessInput.processes.countMissing",
                          deviceName + ".ProcessInput.valid",  
                          deviceName + ".ProcessInput.publicationsValid",                                              
                          sFrontEnd+".alarm",
                          sFrontEnd+".enabled");
    g_bCallbackConnected = (iRes >= 0);    
  }else{
    fecIcemonFec_WidgetDisconnection(g_sWidgetType);
  }
}

/** dpDisconnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, front-end device DP name
@param sFrontEnd input, front-end device name

*/
fecIcemonFec_WidgetDisconnect(string deviceName, string sFrontEnd, string fsmDp)
{
  int iRes;

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec widget IcemonFec "+deviceName, "fecIcemonFec_WidgetDisconnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc)
  {
    // if there were many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // re-use the kind of dpConnect kept in a variable global to the widget set during fecIcemonFec_WidgetConnect
    // and do to the correct dpDisconnect
    iRes = dpDisconnect("fecIcemonFec_WidgetCB", 
                          deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                          deviceName + ".statusInformation.selectedManager",
                          fsmDp + ".fsm.currentState",
                          deviceName + ".:_alert_hdl.._act_state",
                          deviceName + ".ProcessInput.readings.freeMemory.value",
                          deviceName + ".ProcessInput.readings.usedSwap.value",
                          deviceName + ".ProcessInput.readings.cpu.value",                          
                          deviceName + ".ProcessInput.processes.countMissing",
                          deviceName + ".ProcessInput.valid",  
                          deviceName + ".ProcessInput.publicationsValid",  
                          sFrontEnd+".alarm",
                          sFrontEnd+".enabled");
    g_bCallbackConnected = !(iRes >= 0);
  }
}

/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpLocked input, the lock DPE 
@param bLocked input, the lock state 
@param sDpSelectedManager input, the selected manager DPE 
@param sSelectedManager input, the selected manager 
@param sDpInvalid input, the invalid DPE 
@param sDpFESystemIntegrityAlarmValue input, front-end device system integrity alarm DPE 
@param iFESystemIntegrityAlarmValue input, front-end device system integrity alarm value
@param sDpFESystemIntegrityAlarmEnabled input, front-end device system integrity enable DPE
@param bFESystemIntegrityAlarmEnabled input, front-end device system integrity enable value
*/
fecIcemonFec_WidgetCB(string sDpLocked, bool bLocked,
                      string sDpSelectedManager, string sSelectedManager,
                      string sFsmStateDp, string state, 
                      string sAlertStateDp, int alertState,
                      string sFreeMemoryDp, float freeMem,
                      string sUsedSwapDp, float usedSwap,
                      string sCpuDp, float cpu,                      
                      string sMissingProcessesDp, int missingProcessesCount,
                      string sDpInvalid, bool bValid,
                      string sDpPublicationsValid, bool publicationsValid,
                      string sDpFESystemIntegrityAlarmValue, int iFESystemIntegrityAlarmValue, 
                      string sDpFESystemIntegrityAlarmEnabled, bool bFESystemIntegrityAlarmEnabled)
{
  /** use the exec mechanism to allow multiple widget. **/
  string sFunction = "fecIcemonFec_"+g_sWidgetType+"Animation";
  string selectColor, sWarningLetter, sWarningColor;
  bool bSelectVisible;
  
  // 1. set the widget.
  unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, bSelectVisible);

  // if not valid --> N letter and everything blue
  if (!bValid || !publicationsValid){
    sWarningLetter = UN_WIDGET_TEXT_INVALID;
    sWarningColor = "unDataNotValid";
  }  
  else {
// if Front-end problem or problems-> O letter
    if((!bFESystemIntegrityAlarmEnabled) || (iFESystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
    {
      sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
      sWarningColor = "unDataNotValid";
    }
  }
  
  if (alertState == 0)
  {
    txAlertPresence.visible = false;
  }  
  else
  {
    txAlertPresence.visible = true;
  }
  
  if(g_bSystemConnected)
    setMultiValue("WarningText", "text", sWarningLetter, 
                  "WarningText", "foreCol", sWarningColor,
                  "SelectArea", "foreCol", selectColor,
                  "LockBmp", "visible", bSelectVisible,
                  "WidgetArea", "visible", true);


  if (isFunctionDefined(sFunction) && (sFunction != ""))
  {
    string fecDp = unGenericDpFunctions_getDpName(sMissingProcessesDp);
    // set here all the common graphical element value or
    // give to the function all the variables
    execScript("main(string fecDp, string state, float freeMem, float usedSwap, float cpu, int missingProcessesCount, bool bSystemConnected, string sWidgetType, int iFESystemIntegrityAlarmValue, bool bFESystemIntegrityAlarmEnabled) {" + 
               sFunction + "(fecDp, state, freeMem, usedSwap, cpu, missingProcessesCount, bSystemConnected, sWidgetType, iFESystemIntegrityAlarmValue, bFESystemIntegrityAlarmEnabled);}", 
               makeDynString(), fecDp, state, freeMem, usedSwap, cpu, missingProcessesCount, g_bSystemConnected, g_sWidgetType, iFESystemIntegrityAlarmValue, bFESystemIntegrityAlarmEnabled);
  }
}

/** pop-up menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName input, device DP name
@param sDpType input, device type
@param dsAccessOk input, the access control 
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
fecIcemonFec_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
  dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);
  
  unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  if (isFunctionDefined("icemon_addMoonActionsToMen"))
    icemon_addMoonActionsToMen(menuList);
}

/** handle the answer of the popup menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param sDpType input, device type
@param menuList input, the access control 
@param iMenuAnswer input, selected menu value 
*/
fecIcemonFec_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int iMenuAnswer)
{
  string sDeviceName = unGenericDpFunctions_getAlias(deviceName);
  int iUN_POPUPMENU_MIN = UN_POPUPMENU_MIN;
  dyn_string exceptionInfo;
  dyn_string dsTemp, dsSplit;
  int len, i, iMenu, iValue;
  
  // handle unicos menu
  if(iMenuAnswer<iUN_POPUPMENU_MIN)
    unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, iMenuAnswer, exceptionInfo);
  else if (iMenuAnswer == 333){ //LAn DB Info
    icemon_handleMenu(sDeviceName);
  }
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec rigthClick IcemonFec "+deviceName, "fecIcemonFec_HandleMenu", deviceName, sDpType, menuList, iMenuAnswer);
}

/** animate the device Square widget
!!!!! function trigger by exec call, $-param and variable of the widget cannot be used, all the necessary data 
must be given to the function. The use of global var (global keyword declaration) is allowed
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param bSystemConnected input, connection state of the system, true=system connected/false=system not connected
@param sWidgetType input, the type of widget
@param iFESystemIntegrityAlarmValue input, front-end device system integrity alarm value
@param bFESystemIntegrityAlarmEnabled input, front-end device system integrity enable value
*/
fecIcemonFec_SquareAnimation(string fecDp, string state, float freeMem,float usedSwap, float cpu, int missingProcessesCount,  bool bSystemConnected, string sWidgetType, int iFESystemIntegrityAlarmValue, bool bFESystemIntegrityAlarmEnabled)
{
  /*!!!!!!!! all parameters must be given: function launched by exec ***/
  // encapsulate all the setValue or function call with the following if(bSystemConnected)
  string sColor;
  if(bSystemConnected) {
	  string color;
   string alias = dpGetAlias(fecDp + ".");
   fwCU_getStateColor(alias, state, color);

   setMultiValue("txMemory", "text", "Mem: " +(100- floor(freeMem )) + "%",
                  "txSwap", "text", "Swap: " + floor(usedSwap) + "%",
                  "txCPU", "text", "CPU:" + floor(cpu) + "%",                  
                  "txMissing", "text", "Missing:" + missingProcessesCount,
                  "shStateColor", "backCol", color);
  }
}

/** animate the the device Square widget when the system is disconnected 
!!!!! function trigger by exec call, $-param and variable of the widget cannot be used, all the necessary data 
must be given to the function. The use of global var (global keyword declaration) is allowed

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sWidgetType input, the type of widget
*/
fecIcemonFec_SquareDisconnection(string sWidgetType)
{
  // set all the graphical element of the widget
  setMultiValue("Body1", "backCol", "unDataNoAccess",
                "WarningText", "text", "",
                "WidgetArea", "visible", false,
                "SelectArea", "foreCol", "",
                "LockBmp", "visible", false);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "fec widget IcemonFec Square", "fecIcemonFec_SquareDisconnection", sWidgetType);
}

//@}
